 list		p=16f877A	; list directive to define processor
	#include	<p16f877A.inc>	; processor specific variable definitions
	
	__CONFIG _CP_OFF & _WDT_OFF & _BODEN_OFF & _PWRTE_ON & _XT_OSC & _WRT_OFF & _LVP_ON & _CPD_OFF

; '__CONFIG' directive is used to embed configuration data within .asm file.
; The lables following the directive are located in the respective .inc file.
; See respective data sheet for additional information on configuration word.


;***** VARIABLE DEFINITIONS
w_temp		EQU	0x7D		; variable used for context saving 
status_temp	EQU	0x7E		; variable used for context saving
pclath_temp	EQU	0x7F		; variable used for context saving			

 
  cblock	0x20
            count            ;used in loops
            count1			;used in delay routine
			counta			;used in delay routine
			countb			;used in delay routine

            templcd ;temp store for 4 bit mode
  endc 




;**********************************************************************
	ORG     0x000             ; processor reset vector

	nop			  ; nop required for icd
  	goto    Start             ; go to beginning of program


	ORG     0x004             ; interrupt vector location

	movwf   w_temp            ; save off current W register contents
	movf	STATUS,w          ; move status register into W register
	movwf	status_temp       ; save off contents of STATUS register
	movf	PCLATH,w	  ; move pclath register into w register
	movwf	pclath_temp	  ; save off contents of PCLATH register

; isr code can go here or be located as a call subroutine elsewhere

	movf	pclath_temp,w	  ; retrieve copy of PCLATH register
	movwf	PCLATH		  ; restore pre-isr PCLATH register contents
	movf    status_temp,w     ; retrieve copy of STATUS register
	movwf	STATUS            ; restore pre-isr STATUS register contents
	swapf   w_temp,f
	swapf   w_temp,w          ; restore pre-isr W register contents
	retfie                    ; return from interrupt


	

	
;Subroutines:

Init 
        clrw                    ; Zero.
        movwf   PORTB	    	;resets input/output ports
        bsf     STATUS,RP0      ; Select Bank 1
		movlw   b'00000000'    ; Set port B bits 0-3 as outputs b0-b3 connected to lcd  d4-d7 pins
                                ;and b6-b7 connected to RS,E and RW grounded       
        movwf   TRISB        ; Set TRISB register 
        bcf     STATUS,RP0      ; Select Bank 0 
        call	Delay100		;wait for LCD to settle

        retlw 0
	
LCD_Init	
        movlw	0x20			;Set 4 bit mode
		call	LCD_Cmd

        movlw	0x28			;Set display shift
		call	LCD_Cmd

		movlw	0x06			;Set display character mode
		call	LCD_Cmd

		movlw	0x0d			;Set display on/off and cursor command
		call	LCD_Cmd

		call	LCD_Clr			;clear display

		retlw	0x00


LCD_Cmd		
        movwf	templcd
		swapf	templcd,	w	;send upper nibble
		andlw	0x0f			;clear upper 4 bits of W
		movwf	PORTB
		bcf	    PORTB,6	;RS line to 0
		call	Pulse_e			;Pulse the E line high

		movf	templcd,	w	;send lower nibble
		andlw	0x0f			;clear upper 4 bits of W
		movwf	PORTB
		bcf	    PORTB,6	;RS line to 0
		call	Pulse_e			;Pulse the E line high
		call 	Delay5
		retlw	0x00

LCD_Char
     	movwf	templcd
		swapf	templcd,w	;send upper nibble
		andlw	0x0f			;clear upper 4 bits of W
		movwf	PORTB
		bsf	    PORTB,6  	;RS line to 1
		call	Pulse_e			;Pulse the E line high

		movf	templcd,w	;send lower nibble
		andlw	0x0f			;clear upper 4 bits of W
		movwf	PORTB
		bsf	    PORTB,6  	;RS line to 1
		call	Pulse_e			;Pulse the E line high
		call 	Delay5
		retlw	0x00

LCD_Clr		
        movlw	0x01			;Clear display
		call	LCD_Cmd
		retlw	0x00

Delay255	
        movlw	0xff		;delay 255 mS
		goto	d0
Delay100
    	movlw	d'100'		;delay 100mS
		goto	d0
Delay50
		movlw	d'50'		;delay 50mS
		goto	d0
Delay20	
    	movlw	d'20'		;delay 20mS
		goto	d0
Delay5
		movlw	0x05		;delay 5.000 ms (4 MHz clock)
d0		movwf	count1
d1		movlw	0xC7			;delay 1mS
		movwf	counta
		movlw	0x01
		movwf	countb
Delay_0
		decfsz	counta, f
		goto	$+2
		decfsz	countb, f
		goto	Delay_0

		decfsz	count1	,f
		goto	d1
		retlw	0x00

Text
		addwf	PCL, f
		retlw	'H'
		retlw	'e'
		retlw	'l'
		retlw	'l'
		retlw	'o'
		retlw	0x00

Pulse_e		
        bsf	PORTB,7
		nop
		bcf	PORTB,7
		retlw	0x00




EndMessage	
		
Stop		goto	Stop			;endless loop




;Program Start:

Start
		call Init

Main
        clrf count   ;set counter register to zero
message
        movf	count, w		;put counter value in W
        call	Text			;get a character from the text table
        xorlw 0x00  ;is it a zero?
        btfsc STATUS,Z
		goto EndMessage
        call	LCD_Char
        incf	count, f
        goto message


        END	